#!/usr/bin/perl
use strict;
# This script was written by S. Blusk, April 2007
#
# This scripts creates:
# --> a shell script that will submit all the jobs              --> launch_VeloAlign.x
# --> an individual shell script that will be executed in batch --> AlignT2Velo_(...).csh
# --> the corresponding options file                            --> AlignT2Velo_(...).opts


# the job number and max number of jobs                                                                                       
my $n_run = 0;
my $n_maxrun = 100;

# the number of events per job
my $incre = 1000;

my $suffix = "8tev_fieldoff_1000";

my $version = "v31r4";

# the workdir where the script is executed (ADAPT IT TO YOUR CONDITIONS)
my $cmtpath = "/home/sblusk/cmt_Brunelv30";
#my $path = $cmtpath ."/Generation/";
my $path = "/home/sblusk/cmtuser/Brunel_$version/AlignT2Velo/v1r0";

my $cmtconfig = "slc3_ia32_gcc323";

# the directory where the results are stored (ADAPT IT TO YOUR CONDITIONS)
my $pathin = "/raid/data03/user/blusk/Boole/";
my $pathout = "/raid/data03/user/blusk/AlignT2Velo/";


#
# 1. Create the script that launch all the batch jobs
#

my $qsub_filename = "launch_VeloAlign.x";
open(qsub_file,">$qsub_filename") or die "Can't open file $qsub_filename";
while ($n_run <= $n_maxrun) 
{
    my $sh_filename = $path ."/job/scripts/AlignT2Velo_" . $suffix . "_" . $n_run . ".csh";
    my $string = "AT2Velo_" . $n_run;
    print qsub_file "chmod 755 $sh_filename \n"; 
    print qsub_file "qsub -N $string $sh_filename \n"; 
    CreateShellScript($suffix,$incre,$version,$path,$pathout,$n_run);
    CreateOptionsFile($pathin,$suffix,$incre,$n_run);
    $n_run  = $n_run + 1;
}
close(qsub_file);


#
# 2. Create the script that will be used to run gauss in batch 
#

sub CreateShellScript{

# suffix to the filename
    my $suffix = $_[0];
# the event number per run
    my $max_eventN = $_[1];
# the Gauss Version
    my $brunel_version = $_[2];
# the path where the specific options file will be retrieved
    my $path = $_[3];
# the path where the simulated data are stored 
    my $pathout = $_[4];
# the run number
    my $run_number = $_[5];

# Here we define the path and the name of the script file (NOTE that you should have a directory 
# named /scripts/ in your WORKDIR) 
    my $sh_filename = "scripts/AlignT2Velo_" . $suffix ."_" . $run_number . ".csh";

# Same remark than before: options files are store in an /opts/ directory 
    my $options_filename = "AlignT2Velo_" . $suffix . "_" . $run_number . ".opts";
    my $options_completefilename = $path . "/job/opts/" . $options_filename;
 
    my $log_filename = "AlignT2Velo_" . $suffix . "_" . $run_number . ".log";
    my $log_completefilename = $pathout . "logs/" . $log_filename;

    my $dat_filename = "AlignT2Velo_" . $suffix . "_" . $run_number . ".dat";
    my $dat_completefilename = $path . "/results/" . $dat_filename;

    my $results_dir = $path . "/results";
    my $scrdir = "/local1/user/blusk";
    my $topdir = "/local1/user/blusk/AlignT2Velo";
    
    open(sh_file,">$sh_filename") or die "Can't open file $sh_filename";

# set the lhcb environment
    print sh_file "#\$ -S /usr/local/bin/tcsh\n";
# where the job is run
    print sh_file "echo \"Running on HOST \" \$HOST \n"; 
    print sh_file "set MODE = \$CMTCONFIG\n";
    print sh_file "unsetenv CMTPATH\n";
    print sh_file "unalias cp\n";
    print sh_file "mkdir $scrdir \n";
    print sh_file "mkdir $topdir\n";
    print sh_file "mkdir $topdir/$run_number\n";
    print sh_file "set WORKDIR = /local1/user/blusk/AlignT2Velo/$run_number\n";
    print sh_file "set NEV = $max_eventN\n";
    print sh_file "set FIRSTEV = 1\n";
    print sh_file "set NEXP = 5\n";
    print sh_file "set OPTSFILE = '../options/$options_filename'\n";
    print sh_file "set JOBID = $run_number \n";
    print sh_file "echo \"--------------------------------------------------\"\n";
    print sh_file "echo \"Number of Events per Experiment = \" \$NEV \n";
    print sh_file "echo \"First Event in Processing = \" \$FIRSTEV\n";
    print sh_file "echo \"Number of Experiments = \" \$NEXP ";
    print sh_file "echo \"Options File = \" \$OPTSFILE\n";
    print sh_file "echo \"Job Identifier = \"  \$JOBID\n";
    print sh_file "echo \"--------------------------------------------------\" \n";
    print sh_file "#\n";
    print sh_file "echo \"==> Setting up the Environment:\"\n";
    print sh_file "setenv User_release_area ~/cmtuser \n";
    print sh_file "setenvBrunel $brunel_version \n";
    print sh_file "source ~/cmtuser/Brunel_$brunel_version/AlignT2Velo/v1r0/cmt/setup.csh \n";
    print sh_file "cd \$WORKDIR \n";
    print sh_file "mkdir python\n";
    print sh_file "mkdir options\n";
    print sh_file "#\n";
    print sh_file "echo \"===> Updating the CMTPROJECTPATH to:\" \$CMTPROJECTPATH\n";
    print sh_file "setenv CMTPROJECTPATH \${WORKDIR}:\$CMTPROJECTPATH\n";
    print sh_file "#\n";
    print sh_file "echo \"==> Cleaing the directory of old DB, ROOT and DAT files\" \n";
    print sh_file "rm -f python/*.dat\n";
    print sh_file "rm -f python/*.root\n";
    print sh_file "rm -f python/*.db\n";
    print sh_file "rm -f python/*.pyc \n";
    print sh_file "#\n";
    print sh_file "echo \"==> Copying Database Files to local area:\" \n";
    print sh_file "cp \$SQLDDDBROOT/db/DDDB.db python/ \n";
    print sh_file "cp \$SQLDDDBROOT/db/LHCBCOND.db python/ \n";
    print sh_file "#\n";
    print sh_file "echo \"==> Copying scripts to local directory:\" \n";
    print sh_file "cp ~/cmtuser/Brunel_$brunel_version/AlignT2Velo/v1r0/python/*.py python/ \n";
    print sh_file "cp $path/job/v200601_base.opts options/v200601.opts \n";
    print sh_file "cp $options_completefilename options/ \n";
    print sh_file "cd python \n";
    print sh_file "# \n";
    print sh_file "echo \"==> Running the job:\" \n";
    print sh_file "python AlignT2Velo.py -n \$NEV -s \$NEXP -o \$OPTSFILE >& $log_filename\n";
    print sh_file "echo \"==> Job finished\" \n";
    print sh_file "cat rand*.dat > myrandom_\$JOBID.dat\n";
    print sh_file "cp myrandom_\$JOBID.dat $results_dir/random_misalign_$run_number.dat\n";
    print sh_file "#cp fixed_misalign_0.dat $results_dir/fixed_misalign_$run_number.dat\n";
    print sh_file "cp $log_filename $log_completefilename\n";
    print sh_file "cd $topdir\n";
    print sh_file "rm -rf $run_number\n";
    print sh_file "exit 0 \n";
    close(sh_file);
}


#
# 3. Create the corresponding options file (here we load the correct LHCb geometry file)
#    This option file replace the classic Gauss.opts file. It uses a common part, called GaussBasics.opts
#

sub CreateOptionsFile{
# suffix to the filename
    my $pathin = $_[0];
    my $suffix = $_[1];
# maximum number of events
    my $max_eventN = $_[2];
# run number
    my $run_number = $_[3];
    
# lets create the filename and the open the file
    my $filename = "opts/AlignT2Velo_" . $suffix . "_" . $run_number . ".opts";

    my $input_filename = $pathin . "digi/Boole_" . $suffix . "_" . $run_number . ".digi";

    open(option_file,">$filename") or die "Can't open file $filename";

    print option_file "// Standard Options\n";
    print option_file "//--------------------------------------------------------------------------\n";
    print option_file "#include \"v200601.opts\"    // Contains all the common stuff\n";
    print option_file "//--------------------------------------------------------------------------\n";
    print option_file "// Additional Alignment Options\n";
    print option_file "//---------------------------------------------------------------------------\n";
    print option_file "ApplicationMgr.EvtMax     = $max_eventN;\n";
    print option_file "//---------------------------------------------------------------------------\n";
    print option_file "EventSelector.Input = {\n";
    print option_file "\"DATAFILE='PFN:$input_filename' TYP='POOL_ROOT' OPT='READ'\" \n";
    print option_file "};  // My Input Events\n";
    print option_file "//---------------------------------------------------------------------------\n";
    print option_file "// Use these options when field is OFF  \n";
    print option_file "MagneticFieldSvc.UseConstantField = true; // Constant (zero) field  \n";
    print option_file "#include \"\$TRACKSYSROOT/options/NoFieldTracking.opts\"  \n";
    print option_file "// Use this option when VELO is OPEN  \n";
    print option_file "/////#include \"\$TRACKSYSROOT/options/VeloOpenTracking.opts\"  \n";
    print option_file "//---------------------------------------------------------------------------\n";
    print option_file "  \n";
    print option_file "LHCBCOND.ConnectionString = \"sqlite_file:\$PWD/LHCBCOND.db/LHCBCOND\";   \n";
    print option_file "DDDB.ConnectionString = \"sqlite_file:\$PWD/DDDB.db/DDDB\";   \n";
    print option_file "  \n";
    print option_file "TrackPrepareVelo.bestLocation = \"\"; \n";
    close(option_file);
}




