#===================================================================#
# Python algorithm to perform T-Station to VELO relative alignment
#===================================================================#
# @author: Steven Blusk,  sblusk@phy.syr.edu
# @ date: 2007-03-02
#===================================================================#

import gaudimodule
gaudimodule.loaddict('LHCbKernelDict')

import myConstants
import sys
import os
import getopt
import conddbui
from conddbui import cool
import math
from ROOT import *

from AlignmentTools import *

#-------------- Some convenient definitions -----------------
ROOT = gaudimodule.gbl.ROOT
std = gaudimodule.gbl.std
XYZPoint = ROOT.Math.XYZPoint
XYZVector = ROOT.Math.XYZVector
pi = math.pi
#------------------------------------------------------------

def makeDirs():
    print "---------------------------------------"
    print "Creating Conditions Directory Structure"
    print "Conditions/"
    print "Conditions/IT"
    print "Conditions/OT"
    print "Conditions/Velo"
    print "Conditions/IT/Alignment"
    print "Conditions/OT/Alignment"
    print "Conditions/Velo/Alignment"
    print "---------------------------------------"
    os.system("mkdir Conditions")
    os.system("mkdir Conditions/IT")
    os.system("mkdir Conditions/OT")
    os.system("mkdir Conditions/Velo")
    os.system("mkdir Conditions/IT/Alignment")
    os.system("mkdir Conditions/OT/Alignment")
    os.system("mkdir Conditions/Velo/Alignment")
        
class VeloTAlignment:

    def __init__(self, optsFile, name = 'VeloTAlignment'):
        self.name = name
        self.optsFile = optsFile

        appMgr = gaudimodule.AppMgr(outputlevel=3, joboptions=optsFile)

        appMgr.initialize()
        self.gaudi = appMgr

        self.evt = appMgr.evtsvc()
        self.det = appMgr.detsvc()
        self.sel = appMgr.evtsel()

        self.updMgrSvc = appMgr.service('UpdateManagerSvc','IUpdateManagerSvc')
        self.magFieldSvc = appMgr.service('MagneticFieldSvc','IMagneticFieldSvc')
        self.condAccSvc = appMgr.service('LHCBCOND','ICondDBAccessSvc')
        self.xmlParserSvc = appMgr.service('XmlParserSvc','IXmlParserSvc')


        self.OTDetElem = self.det['/dd/Structure/LHCb/AfterMagnetRegion/T/OT']
        self.ITDetElem = self.det['/dd/Structure/LHCb/AfterMagnetRegion/T/IT']
        self.myITCond = self.ITDetElem.geometry().alignmentCondition()
        self.myOTCond = self.OTDetElem.geometry().alignmentCondition()
        self.DTD = '"../../structure.dtd"'

        
    def StoreOffsetsInDB(self):

        myITDetElem = self.ITDetElem
        myOTDetElem = self.OTDetElem
        myDTD       = self.DTD

        myITXml = makeAlignments(myITDetElem , 'IT' , ' ' , myDTD)
        myOTXml = makeAlignments(myOTDetElem , 'OT' , ' ' , myDTD)

        iver = 1
# Write (mis)Alignment Files
        it_output_file = 'Conditions/IT/Alignment/Detectors_'+str(iver)+'.xml'
        ot_output_file = 'Conditions/OT/Alignment/Elements_'+str(iver)+'.xml'
        writeAlignmentFile(it_output_file, myITXml)
        writeAlignmentFile(ot_output_file, myOTXml)
    
# write MainCatalog
        makeMainCatalog(iver)

# Make VeloCatalog, ITCatalog, and OTCatalog
        makeVeloAlignmentCatalog(iver)
        makeITAlignmentCatalog(iver)
        makeOTAlignmentCatalog(iver)

        self.updMgrSvc.invalidate(self.myITCond)
        self.updMgrSvc.newEvent()
        alignCond = myITDetElem.geometry().alignmentCondition()
        thePositions = alignCond.paramAsDoubleVect("dPosXYZ")
        print "---> The XYZ positions are: ",thePositions[0],thePositions[1],thePositions[2]
        
        print "---------------- Finished ----------------------"

    def InjectMisalignment(self,t,r):
        myConstants.tglo_trans = t
        myConstants.tglo_rot = r
        self.StoreOffsetsInDB()
    


if __name__ == '__main__' :

    makeDirs()
    
    optsFile = '../options/createAlignment.opts'

    job = VeloTAlignment(optsFile)

    gaudi=job.gaudi

    # simulate a fixed misalignments, t = translations, r=rotations
    t = [ 5.0, 0.0, 0.0]
    r = [0.0, 0.0, 0.0]
    job.InjectMisalignment(t,r)




        

    
